@@ -19,7 +19,7 @@
 #include "re.h"
 
 
-#define TMAIL_VERSION "0.10.3"
+#define TMAIL_VERSION "0.10.5"
 
 static VALUE MailScanner;
 static VALUE ScanError;
@@ -141,7 +141,7 @@ mails_debug_set(self, flag)
 #define IS_JCHAR(ch)      ismbchar(ch)
 
 
-/* I know this implement is agree, but usually useful. */
+/* I know this implement is ugly, but usually useful. */
 
 /* skip until "\e(B" (us-ascii) */
 static void
@@ -393,9 +393,23 @@ debug_print(sc, sym, val)
     VALUE s;
 
     s = rb_funcall(sym, rb_intern("inspect"), 0),
-    printf("%7ld %-10s %s\n", (unsigned long)(sc->pend - sc->p),
-                              RSTRING(s)->ptr,
-                              RSTRING(val)->ptr);
+    printf("%7ld %-10s token=<%s>\n",
+           (unsigned long)(sc->pend - sc->p),
+           RSTRING(s)->ptr,
+           RSTRING(val)->ptr);
+}
+
+#define D(m) do { if (sc->flags & MODE_DEBUG) {m;} } while (0)
+
+static void
+pass_token(sc, sym, tok, arr)
+    struct scanner *sc;
+    VALUE sym, tok, arr;
+{
+    D(debug_print(sc, sym, tok));
+    rb_ary_store(arr, 0, sym);
+    rb_ary_store(arr, 1, tok);
+    rb_yield(arr);
 }
 
 static VALUE
@@ -404,27 +418,19 @@ mails_scan(self)
 {
     struct scanner *sc;
     VALUE arr;
-    int debug;
-
-#define PASS(s,v) \
-    if (debug) debug_print(sc,s,v);\
-    rb_ary_store(arr, 0, s);\
-    rb_ary_store(arr, 1, v);\
-    rb_yield(arr)
-
-#define D(m) if (debug) puts(m)
 
+#define PASS(s,v) pass_token(sc,s,v,arr)
     GET_SCANNER(self, sc);
     if (!sc->p) {
         rb_raise(ScanError, "Mails#scan called before reset");
     }
     arr = rb_assoc_new(Qnil, Qnil);
-    debug = sc->flags & MODE_DEBUG;
 
     while (sc->p < sc->pend) {
-        D("new loop");
+        D(puts("new loop"));
+        D(printf("char='%c'\n", *sc->p));
         if (IS_LWSP(*sc->p)) {
-            D("lwsp");
+            D(puts("lwsp"));
             skip_lwsp(sc);
             if (sc->p >= sc->pend)
                 break;
@@ -434,7 +440,7 @@ mails_scan(self)
             if (IS_TOKENCHAR(*sc->p) ||
                     (ISO2022_MODE_P(sc) && (*sc->p == ESC)) ||
                     IS_JCHAR(*sc->p)) {
-                D("token");
+                D(puts("token"));
                 PASS(tok_token, scan_token(sc));
                 continue;
             }
@@ -444,7 +450,7 @@ mails_scan(self)
                     (ISO2022_MODE_P(sc) && (*sc->p == ESC)) ||
                     IS_JCHAR(*sc->p)) {
                 VALUE tmp;
-                D("atom");
+                D(puts("atom"));
                 tmp = scan_atom(sc);
                 PASS(atomsym(sc, tmp), tmp);
                 continue;
@@ -452,23 +458,24 @@ mails_scan(self)
         }
 
         if (*sc->p == '"') {
-            if (debug) puts("quoted");
+            D(puts("quoted"));
             PASS(tok_quoted, scan_quoted_word(sc));
+            D(puts("quoted"));
         }
         else if (*sc->p == '(') {
             VALUE c;
-            D("comment");
+            D(puts("comment"));
             c = scan_comment(sc);
             if (! NIL_P(sc->comments))
                 rb_ary_push(sc->comments, c);
         }
         else if (*sc->p == '[') {
-            D("domlit");
+            D(puts("domlit"));
             PASS(tok_domlit, scan_domain_literal(sc));
         }
         else {
             VALUE ch;
-            D("char");
+            D(puts("char"));
             ch = rb_str_new(sc->p, 1);
             sc->p++;
             PASS(ch, ch);
