@@ -53,7 +53,7 @@ module TMail
     attr_reader :port
 
     def inspect
-      "\#<#{type} port=#{@port.inspect} bodyport=#{@body_port.inspect}>"
+      "\#<#{self.class} port=#{@port.inspect} bodyport=#{@body_port.inspect}>"
     end
 
 
@@ -66,6 +66,7 @@ module TMail
     include StrategyInterface
 
     def write_back( eol = "\n", charset = 'e' )
+      parse_body
       @port.wopen {|stream| encoded eol, charset, stream }
     end
 
@@ -79,7 +80,7 @@ module TMail
           end
           f.puts sep
 
-          body_port.ropen {|r|
+          body_port().ropen {|r|
               f.write r.read
           }
       }
@@ -88,7 +89,7 @@ module TMail
     private
 
     def canonical( name )
-      name.split(/-/).collect {|s| s.capitalize }.join('-')
+      name.split(/-/).map {|s| s.capitalize }.join('-')
     end
 
     def with_multipart_encoding( strategy, f )
@@ -253,30 +254,9 @@ module TMail
       @header.key? key.downcase
     end
 
-    alias include? key?   # obsolete
-    alias has_key? key?   # obsolete
-
-
-    # obsolete
-    def values
-      ret = []
-      each_field {|v| ret.push v }
-      ret
-    end
-
-    # obsolete
-    def value?( val )
-      HeaderField === val or return false
-
-      [ @header[val.name.downcase] ].flatten.include? val
-    end
-
-    # obsolete
-    alias has_value? value?
-
 
     def select( *args )
-      args.collect {|k| @header[k.downcase] }.flatten
+      args.map {|k| @header[k.downcase] }.flatten
     end
 
     alias indexes select
@@ -350,7 +330,7 @@ module TMail
     end
 
     def each( &block )
-      body_port.ropen {|f| f.each(&block) }
+      body_port().ropen {|f| f.each(&block) }
     end
 
     def body
@@ -418,35 +398,26 @@ module TMail
       is_sep = /\A--#{Regexp.quote bound}(?:--)?[ \t]*(?:\n|\r\n|\r)/
       lastbound = "--#{bound}--"
 
-      f = nil
-      n = 1
+      ports = [ @config.new_preamble_port(self) ]
       begin
-        ports = []
-        ports.push @config.new_preamble_port(self)
-        f = ports[-1].wopen
-
-        while line = src.gets do    # no each !
+        f = ports.last.wopen
+        while line = src.gets do
           if is_sep === line then
             f.close
-            line.strip!
-            if line == lastbound then
-              break
-            else
-              ports.push @config.new_part_port(self)
-              n += 1
-              f = ports[-1].wopen
-            end
+            break if line.strip == lastbound
+            ports.push @config.new_part_port(self)
+            f = ports.last.wopen
           else
             f << line
           end
         end
-        @epilogue = src.read
+        @epilogue = (src.read || '')
       ensure
         f.close if f and not f.closed?
       end
 
       @body_port = ports.shift
-      @parts = ports.collect {|p| type().new p }
+      @parts = ports.map {|p| self.class.new(p, @config) }
     end
 
   end   # class Mail
